@@ -1,6 +1,8 @@
 """Error objects"""
+
 import dataclasses
-from typing import Callable, Mapping, Union, Any, Type
+from typing import Union, Any, Type
+from collections.abc import Callable, Mapping
 from functools import partial
 from contextlib import AbstractContextManager
 
@@ -66,7 +68,7 @@ class ModuleNotFoundIgnore:
 
 
 OnErrorCallback = Union[None, str, Callable[[], Any]]
-ExceptionType = Type[BaseException]
+ExceptionType = type[BaseException]
 
 
 def log_and_return(msg, logger=print):
@@ -208,9 +210,9 @@ class HandleExceptions(AbstractContextManager):
             raise
 
     def exited_with_handled_exception(self):
-        return hasattr(self, 'exit_value')
+        return hasattr(self, "exit_value")
 
     def initialize(self):
-        if hasattr(self, 'exit_value'):
-            delattr(self, 'exit_value')
+        if hasattr(self, "exit_value"):
+            delattr(self, "exit_value")
         self.exited_with_exception = None
