@@ -7,13 +7,13 @@ wrapped function itself.
 Output value tranformers can be conditioned on argument value and the wrapped function.
 
 """
+
 from dataclasses import dataclass
 from typing import (
-    Mapping,
-    Callable,
     Optional,
     TypedDict,
 )
+from collections.abc import Mapping, Callable
 from inspect import signature, Parameter
 from pickle import dumps
 
@@ -31,13 +31,13 @@ _TypedDictMeta = type(TypedDict)
 
 # monkey patching WRAPPER_ASSIGNMENTS to get "proper" wrapping (adding defaults and kwdefaults
 wrapper_assignments = (
-    '__module__',
-    '__name__',
-    '__qualname__',
-    '__doc__',
-    '__annotations__',
-    '__defaults__',
-    '__kwdefaults__',
+    "__module__",
+    "__name__",
+    "__qualname__",
+    "__doc__",
+    "__annotations__",
+    "__defaults__",
+    "__kwdefaults__",
 )
 
 update_wrapper = functools.update_wrapper
@@ -68,12 +68,12 @@ class IoTrans:
             func
         )  # Todo: Want empty mapping as default (use frozendict or __post_init__?)
         def wrapped_func(*args, **kwargs):
-            original_kwargs = sig.extract_kwargs(*args, **kwargs)
+            original_kwargs = sig.map_arguments_from_variadics(*args, **kwargs)
             new_kwargs = {
                 argname: self.in_val_trans(argval, argname, func)
                 for argname, argval in original_kwargs.items()
             }
-            new_args, new_kwargs = sig.args_and_kwargs_from_kwargs(new_kwargs)
+            new_args, new_kwargs = sig.mk_args_and_kwargs(new_kwargs)
             return self.out_trans(func(*new_args, **new_kwargs), func)
 
         return wrapped_func
@@ -167,8 +167,8 @@ class JSONAnnotAndDfltIoTrans(AnnotAndDfltIoTrans):
                 or isinstance(param.default, bool)
             ):
                 return cast_to_jdict(argval)
-            if hasattr(param.annotation, '__iter__') or hasattr(
-                param.default, '__iter__'
+            if hasattr(param.annotation, "__iter__") or hasattr(
+                param.default, "__iter__"
             ):
                 return cast_to_list(argval)
         return super().in_val_trans(argval, argname, func)
@@ -207,8 +207,10 @@ class JSONAnnotAndDfltIoTrans(AnnotAndDfltIoTrans):
 class TypedBasedOutIoTrans(IoTrans):
     """Transform output according to it's type."""
 
-    trans_func_for_type: Mapping = ()  # Todo: Want empty mapping as default (use frozendict or __post_init__?)
-    dflt_trans_func: Optional[Callable] = None
+    trans_func_for_type: (
+        Mapping
+    ) = ()  # Todo: Want empty mapping as default (use frozendict or __post_init__?)
+    dflt_trans_func: Callable | None = None
 
     def out_trans(self, argval, func):
         for typ in self.trans_func_for_type:
@@ -255,18 +257,18 @@ def cast_to_jdict(value):
         value = value.strip()
         if value:
             first_char = value[0]
-            if first_char in {'[', '{'}:
+            if first_char in {"[", "{"}:
                 return json.loads(value)
-            elif value in ['true', 'false', 'null']:
+            elif value in ["true", "false", "null"]:
                 return json.loads(value)
             elif os.path.isfile(value):
                 return json.load(value)
             else:
                 return json.loads(
-                    '[' + value + ']'
+                    "[" + value + "]"
                 )  # wrap in brackets and call json.loads
         else:
-            return ''
+            return ""
     else:
         return value
 
@@ -292,7 +294,7 @@ def cast_to_list(value):
         value = cast_to_jdict(value)
         assert isinstance(value, list)
         return value
-    elif hasattr(value, 'tolist'):  # meant for numpy arrays
+    elif hasattr(value, "tolist"):  # meant for numpy arrays
         # what other potential attributes to check for?
         return value.tolist()
     else:
