@@ -3,7 +3,6 @@
 from typing import (
     NewType,
     Optional,
-    Iterable,
     Protocol,
     Any,
     runtime_checkable,
@@ -11,6 +10,7 @@ from typing import (
     Literal,
     get_origin,
 )
+from collections.abc import Iterable
 
 
 from inspect import signature
@@ -61,8 +61,8 @@ def _value_is_in_literal(value, literal):
 def _validate_that_value_is_in_literal(name, value, literal):
     if not _value_is_in_literal(value, literal):
         error = ValueError(
-            f'{value} is an invalid value for {name}. '
-            f'Values should be one of the following: {literal.__args__}'
+            f"{value} is an invalid value for {name}. "
+            f"Values should be one of the following: {literal.__args__}"
         )
         error.allowed_values = literal.__args__
         error.input_value = value
@@ -118,8 +118,8 @@ def iterable_to_literal(iterable: Iterable):
 def new_type(
     name,
     tp,
-    doc: Optional[str] = None,
-    aka: Optional[Iterable] = None,
+    doc: str | None = None,
+    aka: Iterable | None = None,
     assign_to_globals=False,
 ):
     """
@@ -151,9 +151,9 @@ def new_type(
     """
     new_tp = NewType(name, tp)
     if doc is not None:
-        setattr(new_tp, '__doc__', doc)
+        setattr(new_tp, "__doc__", doc)
     if aka is not None:
-        setattr(new_tp, '_aka', set(aka))
+        setattr(new_tp, "_aka", set(aka))
     if assign_to_globals:
         globals()[
             name
@@ -228,8 +228,8 @@ class HasAttrs:
 def is_a_new_type(typ):
     return (
         callable(typ)
-        and getattr(typ, '__qualname__', '').startswith('NewType')
-        and hasattr(typ, '__supertype__')
+        and getattr(typ, "__qualname__", "").startswith("NewType")
+        and hasattr(typ, "__supertype__")
     )
 
 
@@ -252,7 +252,7 @@ def is_callable_kind(typ):
     """
     if is_a_new_type(typ):
         return is_callable_kind(typ.__supertype__)
-    return typ_name(typ) == 'Callable'
+    return typ_name(typ) == "Callable"
     # Also possible: typ.mro()[0] == __import__('collections.abc').Callable
 
 
@@ -282,22 +282,148 @@ def input_and_output_types(typ: type):
     """
     if is_a_new_type(typ):
         return input_and_output_types(typ.__supertype__)
-    assert is_callable_kind(typ), f'Is not a typing.Callable kind: {typ}'
+    assert is_callable_kind(typ), f"Is not a typing.Callable kind: {typ}"
     typ_args = get_args(typ)
-    assert len(typ_args) > 0, f'Can only be used on a Callable[[...],...] kind: {typ}'
+    assert len(typ_args) > 0, f"Can only be used on a Callable[[...],...] kind: {typ}"
     return typ_args[0], typ_args[1]
 
 
 def dot_string_of_callable_typ(typ):
     input_types, output_type = input_and_output_types(typ)
     return (
-        ','.join(map(typ_name, input_types))
-        + f' -> {typ_name(typ)} -> '
+        ",".join(map(typ_name, input_types))
+        + f" -> {typ_name(typ)} -> "
         + typ_name(output_type)
     )
 
 
-def dot_strings_of_callable_types(*typs, func_shape='box'):
+def dot_strings_of_callable_types(*typs, func_shape="box"):
     for typ in typs:
         yield dot_string_of_callable_typ(typ)
         yield f'{typ_name(typ)} [shape="{func_shape}"]'
+
+
+# --------------------------------------------------------------------------------------
+# Misc
+
+from typing import Any, Dict, Optional, KT
+from collections.abc import Callable, Iterator, Iterable
+
+# Define a type alias for clarity in the code
+ObjectType = Any
+
+
+class ObjectClassifier:
+    """
+    A general-purpose classifier for objects based on a set of verifying functions.
+
+    Each "verifier" checks whether an object belongs to a certain kind (category).
+
+    Example usage:
+
+    >>> from typing import Mapping, Iterable
+    >>>
+    >>> obj = "test"
+    >>> isa = lambda typ: lambda obj: isinstance(obj, typ)
+    >>> verifiers = {
+    ...     'str': isa(str),
+    ...     'mapping': isa(Mapping),
+    ...     'iterable': isa(Iterable)
+    ... }
+    >>> classifier = ObjectClassifier(verifiers)
+
+    Check if the object matches any kind
+
+    >>> classifier.matches(obj)
+    True
+
+    Check if the object matches a specific kind
+
+    >>> classifier.matches(obj, 'str')
+    True
+    >>> classifier.matches(obj, 'mapping')
+    False
+
+    Get all matches
+
+    >>> classifier.all_matches(obj)
+    {'str': True, 'mapping': False, 'iterable': True}
+
+    Find all matching kinds
+
+    >>> list(classifier.matching_kinds(obj))
+    ['str', 'iterable']
+
+    Find the first matching kind (default is to ensure uniqueness, which will fail here)
+
+    >>> classifier.matching_kind(obj)
+    Traceback (most recent call last):
+      ...
+    ValueError: Multiple matches found: ['str', 'iterable']
+
+    Find the first matching kind without uniqueness check
+
+    >>> classifier.matching_kind(obj, assert_unique=False)
+    'str'
+
+    """
+
+    def __init__(self, verifiers: dict[KT, Callable[[ObjectType], bool]]):
+        """
+        Initialize with a dictionary of verifiers. Each verifier is a function
+        that returns True or False based on the object's classification.
+
+        :param verifiers: A dictionary mapping kind names (keys) to verifying functions (values).
+        """
+        self.verifiers = verifiers
+
+    def matches(self, obj: ObjectType, kind: KT | None = None) -> bool:
+        """
+        Returns True if the object matches the given kind, or matches any kind
+        if kind is None.
+
+        :param obj: The object to classify.
+        :param kind: The specific kind (verifier key) to check.
+        :return: True if the object matches the given or any kind.
+        """
+        if kind is None:
+            # Check against all kinds if kind is not specified
+            return any(verifier(obj) for verifier in self.verifiers.values())
+        # Check for the specific kind
+        return self.verifiers.get(kind, lambda x: False)(obj)
+
+    def all_matches(self, obj: ObjectType) -> dict[KT, bool]:
+        """
+        Returns a dictionary indicating if the object matches each kind.
+
+        :param obj: The object to classify.
+        :return: A dictionary with kind names as keys and True/False as values.
+        """
+        return {kind: verifier(obj) for kind, verifier in self.verifiers.items()}
+
+    def matching_kind(
+        self, obj: ObjectType, *, assert_unique: bool = True
+    ) -> KT | None:
+        """
+        Returns the first kind that matches the object. If assert_unique is True,
+        it asserts that only one match exists. Optionally, it can return the value instead of the key.
+
+        :param obj: The object to classify.
+        :param assert_unique: Ensures only one kind matches, if True.
+        :return: The key of the first matching kind, or None if no match.
+        """
+        matches = list(self.matching_kinds(obj))
+        if assert_unique and len(matches) > 1:
+            raise ValueError(f"Multiple matches found: {matches}")
+        return matches[0] if matches else None
+
+    def matching_kinds(self, obj: ObjectType) -> Iterator[KT]:
+        """
+        Returns an iterator of kinds that match the object.
+
+        :param obj: The object to classify.
+        :return: An iterator of matching kinds.
+        """
+        for kind, verifier in self.verifiers.items():
+            if verifier(obj):
+                yield kind
