@@ -149,11 +149,13 @@ could_be_float
 >>>
 
 """
+
 from itertools import chain
 from dataclasses import dataclass
 from i2.util import LiteralVal, mk_sentinel
 
-from typing import TypeVar, Tuple, Iterable, Callable, Mapping, MutableMapping, Any
+from typing import TypeVar, Tuple, Any
+from collections.abc import Iterable, Callable, Mapping, MutableMapping
 
 
 def return_sentinel(obj: Any, sentinel: Any = None):
@@ -270,7 +272,7 @@ class KeyFuncMapping(DelegateToMappingAttrMixin, MutableMapping):
     default_factory: Callable = return_sentinel
 
     def __post_init__(self):
-        assert callable(self.default_factory), f'{self.default_factory} is not callable'
+        assert callable(self.default_factory), f"{self.default_factory} is not callable"
 
     # Note: An earlier version used __getitem__ itself instead of __call__ as the
     #  obj routing method, but the misalignment between the keys listed by __iter__ and
@@ -297,17 +299,17 @@ class KeyFuncMapping(DelegateToMappingAttrMixin, MutableMapping):
         self_type = type(self)
         assert callable(
             other
-        ), f'Can only add a callable (usually a {self_type} itself) to a {self_type}'
+        ), f"Can only add a callable (usually a {self_type} itself) to a {self_type}"
         return self_type(self.mapping, self.key, default_factory=other)
 
 
 # --------------------------------------------------------------------------------------
 # A proposal for a routing forest OO backend and convenience mini-language
-Obj = TypeVar('Obj')
-Output = TypeVar('Output')
+Obj = TypeVar("Obj")
+Output = TypeVar("Output")
 Cond = Callable[[Obj], bool]
 Then = Callable[[Obj], Output]
-Rule = Tuple[Cond, Then]
+Rule = tuple[Cond, Then]
 Rules = Iterable[Rule]
 
 
@@ -327,18 +329,18 @@ def _default_mini_lang(x):
             return SwitchCaseNode(*map(_default_mini_lang, x))
         else:
             raise ValueError(
-                f'If a tuple, element must be a `(cond, then)` pair, '
-                f'or a (switch, case, default) triple, or a `Literal`. Was: {x}'
+                f"If a tuple, element must be a `(cond, then)` pair, "
+                f"or a (switch, case, default) triple, or a `Literal`. Was: {x}"
             )
     elif isinstance(x, dict):
         keys = set(x)
         if 2 <= len(keys) <= 3:
-            if {'cond', 'then'}.issubset(keys):
+            if {"cond", "then"}.issubset(keys):
                 return CondNode(**{k: _default_mini_lang(v) for k, v in x.items()})
-            elif {'switch', 'case'}.issubset(keys) and keys == {
-                'switch',
-                'case',
-                'default',
+            elif {"switch", "case"}.issubset(keys) and keys == {
+                "switch",
+                "case",
+                "default",
             }:
                 return SwitchCaseNode(
                     **{k: _default_mini_lang(v) for k, v in x.items()}
@@ -346,7 +348,7 @@ def _default_mini_lang(x):
             else:
                 raise ValueError(
                     "keys should be 'switch', 'case' and optionally 'default'. "
-                    f'Were: {keys}'
+                    f"Were: {keys}"
                 )
         else:
             raise ValueError(
@@ -364,7 +366,7 @@ class RoutingNode:
     yielding an iterable or a final value"""
 
     def __call__(self, obj) -> Iterable:
-        raise NotImplementedError('You should implement this.')
+        raise NotImplementedError("You should implement this.")
 
     @staticmethod
     def from_object(x, mini_lang=_default_mini_lang):
@@ -449,7 +451,7 @@ class RoutingForest(RoutingNode):
         yield from chain(*self.cond_nodes)
 
 
-Feature = TypeVar('Feature')
+Feature = TypeVar("Feature")
 Featurizer = Callable[[Obj], Feature]
 FeatCondThenMap = Mapping[Feature, Any]
 
@@ -481,7 +483,7 @@ class FeatCondNode(RoutingNode):
     """
 
     feat: Featurizer
-    feat_cond_thens: Iterable[Tuple[Callable[[Feature], bool], Any]]
+    feat_cond_thens: Iterable[tuple[Callable[[Feature], bool], Any]]
 
     @classmethod
     def from_feature_val_map(cls, feat, feat_cond_thens: FeatCondThenMap):
@@ -564,7 +566,7 @@ class FeatCondNode(RoutingNode):
 #         self.feat_cond_map = feat_cond_map
 
 
-NoDefault = type('NoDefault', (object,), {})
+NoDefault = type("NoDefault", (object,), {})
 NO_DFLT = NoDefault()
 
 
@@ -652,16 +654,16 @@ def test_routing_forest():
 
     assert could_be_int(30)
     assert could_be_int(30.3)
-    assert not could_be_int('30.2')
-    assert not could_be_int('nope')
+    assert not could_be_int("30.2")
+    assert not could_be_int("nope")
 
     assert could_be_float(30)
     assert could_be_float(30.3)
-    assert could_be_float('30.2')
-    assert not could_be_float('nope')
+    assert could_be_float("30.2")
+    assert not could_be_float("nope")
 
-    assert could_be_int('30.2') is False
-    assert could_be_float('30.2') is True
+    assert could_be_int("30.2") is False
+    assert could_be_float("30.2") is True
 
     st = RoutingForest(
         [
@@ -671,7 +673,7 @@ def test_routing_forest():
                     [
                         CondNode(
                             cond=lambda x: int(x) >= 10,
-                            then=FinalNode('More than a digit'),
+                            then=FinalNode("More than a digit"),
                         ),
                         CondNode(
                             cond=lambda x: (int(x) % 2) == 1,
@@ -680,39 +682,39 @@ def test_routing_forest():
                     ]
                 ),
             ),
-            CondNode(cond=could_be_float, then=FinalNode('could be seen as a float')),
+            CondNode(cond=could_be_float, then=FinalNode("could be seen as a float")),
         ]
     )
-    assert list(st('nothing I can do with that')) == []
-    assert list(st(8)) == ['could be seen as a float']
-    assert list(st(9)) == ["That's odd!", 'could be seen as a float']
-    assert list(st(10)) == ['More than a digit', 'could be seen as a float']
+    assert list(st("nothing I can do with that")) == []
+    assert list(st(8)) == ["could be seen as a float"]
+    assert list(st(9)) == ["That's odd!", "could be seen as a float"]
+    assert list(st(10)) == ["More than a digit", "could be seen as a float"]
     assert list(st(11)) == [
-        'More than a digit',
+        "More than a digit",
         "That's odd!",
-        'could be seen as a float',
+        "could be seen as a float",
     ]
 
     rf = RoutingForest(
         [
             SwitchCaseNode(
                 switch=lambda x: x % 5,
-                cases={0: FinalNode('zero_mod_5'), 1: FinalNode('one_mod_5')},
-                default=FinalNode('default_mod_5'),
+                cases={0: FinalNode("zero_mod_5"), 1: FinalNode("one_mod_5")},
+                default=FinalNode("default_mod_5"),
             ),
             SwitchCaseNode(
                 switch=lambda x: x % 2,
-                cases={0: FinalNode('even'), 1: FinalNode('odd')},
-                default=FinalNode('that is not an int'),
+                cases={0: FinalNode("even"), 1: FinalNode("odd")},
+                default=FinalNode("that is not an int"),
             ),
         ]
     )
 
-    assert list(rf(5)) == ['zero_mod_5', 'odd']
-    assert list(rf(6)) == ['one_mod_5', 'even']
-    assert list(rf(7)) == ['default_mod_5', 'odd']
-    assert list(rf(8)) == ['default_mod_5', 'even']
-    assert list(rf(10)) == ['zero_mod_5', 'even']
+    assert list(rf(5)) == ["zero_mod_5", "odd"]
+    assert list(rf(6)) == ["one_mod_5", "even"]
+    assert list(rf(7)) == ["default_mod_5", "odd"]
+    assert list(rf(8)) == ["default_mod_5", "even"]
+    assert list(rf(10)) == ["zero_mod_5", "even"]
 
     # testing default mini-language #####################################################
 
@@ -721,41 +723,41 @@ def test_routing_forest():
             (  # CondNode
                 could_be_int,
                 [  # RoutingForest
-                    (lambda x: int(x) >= 10, FinalNode('More than a digit')),
+                    (lambda x: int(x) >= 10, FinalNode("More than a digit")),
                     (lambda x: (int(x) % 2) == 1, FinalNode("That's odd!")),
                 ],
             ),
-            (could_be_float, FinalNode('could be seen as a float')),
+            (could_be_float, FinalNode("could be seen as a float")),
         ]
     )
 
-    assert list(st2('nothing I can do with that')) == []
-    assert list(st2(8)) == ['could be seen as a float']
-    assert list(st2(9)) == ["That's odd!", 'could be seen as a float']
-    assert list(st2(10)) == ['More than a digit', 'could be seen as a float']
+    assert list(st2("nothing I can do with that")) == []
+    assert list(st2(8)) == ["could be seen as a float"]
+    assert list(st2(9)) == ["That's odd!", "could be seen as a float"]
+    assert list(st2(10)) == ["More than a digit", "could be seen as a float"]
     assert list(st2(11)) == [
-        'More than a digit',
+        "More than a digit",
         "That's odd!",
-        'could be seen as a float',
+        "could be seen as a float",
     ]
 
     rf2 = RoutingForest(
         [
             (
                 lambda x: x % 5,
-                LiteralVal({0: FinalNode('zero_mod_5'), 1: FinalNode('one_mod_5')}),
-                FinalNode('default_mod_5'),
+                LiteralVal({0: FinalNode("zero_mod_5"), 1: FinalNode("one_mod_5")}),
+                FinalNode("default_mod_5"),
             ),
             (
                 lambda x: x % 2,
-                LiteralVal({0: FinalNode('even'), 1: FinalNode('odd')}),
-                FinalNode('that is not an int'),
+                LiteralVal({0: FinalNode("even"), 1: FinalNode("odd")}),
+                FinalNode("that is not an int"),
             ),
         ]
     )
 
-    assert list(rf2(5)) == ['zero_mod_5', 'odd']
-    assert list(rf2(6)) == ['one_mod_5', 'even']
-    assert list(rf2(7)) == ['default_mod_5', 'odd']
-    assert list(rf2(8)) == ['default_mod_5', 'even']
-    assert list(rf2(10)) == ['zero_mod_5', 'even']
+    assert list(rf2(5)) == ["zero_mod_5", "odd"]
+    assert list(rf2(6)) == ["one_mod_5", "even"]
+    assert list(rf2(7)) == ["default_mod_5", "odd"]
+    assert list(rf2(8)) == ["default_mod_5", "even"]
+    assert list(rf2(10)) == ["zero_mod_5", "even"]
