@@ -6,13 +6,10 @@ from typing import (
     List,
     Any,
     Union,
-    Callable,
-    Iterator,
     Tuple,
     Optional,
-    Iterable,
-    Mapping,
 )
+from collections.abc import Callable, Iterator, Iterable, Mapping
 from inspect import signature
 
 from i2.signatures import KO, PK, PO, VP, VK, var_param_kinds
@@ -20,15 +17,15 @@ from i2.signatures import _empty
 from i2.signatures import ParamsAble, Sig, ensure_param, SignatureAble
 
 ParameterAble = Union[int, Parameter, str]
-ParamsAble_ = Union[ParamsAble, str, List[int]]
+ParamsAble_ = Union[ParamsAble, str, list[int]]
 
 
 def _is_valid_arg_for_sig(x):
     return (
         isinstance(x, (Callable, Signature))
         or isinstance(x, str)
-        and x.startswith('(')
-        and x.endswith(')')
+        and x.startswith("(")
+        and x.endswith(")")
     )
 
 
@@ -50,13 +47,13 @@ def generate_params(params: ParamsAble_):
         for i, spec in enumerate(params):
             if isinstance(spec, int):
                 kind = spec
-                yield Parameter(f'a{kind}{i}', kind=kind)
+                yield Parameter(f"a{kind}{i}", kind=kind)
             elif isinstance(spec, Parameter):
                 param = spec
                 yield param
             elif isinstance(spec, str) and spec.isnumeric():
                 kind = int(spec)
-                yield Parameter(f'a{kind}{i}', kind=kind)
+                yield Parameter(f"a{kind}{i}", kind=kind)
             else:
                 try:
                     yield ensure_param(spec)
@@ -85,7 +82,7 @@ def params_to_arg_name_and_val(params: ParamsAble_):
         if param.kind == Parameter.VAR_POSITIONAL:
             val = (i, -i)
         elif param.kind == Parameter.VAR_KEYWORD:
-            val = {param.name: i, param.name + '_': -i}
+            val = {param.name: i, param.name + "_": -i}
         else:
             val = i
         yield (param.name, val)
@@ -113,15 +110,15 @@ def inject_defaults(params: ParamsAble_, defaults: dict):
 
 
 def _str_of_call_args(_call_kwargs: dict):
-    return ', '.join(f'{k}={v}' for k, v in _call_kwargs.items())
+    return ", ".join(f"{k}={v}" for k, v in _call_kwargs.items())
 
 
 def _params_to_name(params):
-    return 'f' + ''.join(str(int(p.kind)) for p in params)
+    return "f" + "".join(str(int(p.kind)) for p in params)
 
 
 def mk_func_from_params(
-    params: ParamsAble = '00111234',
+    params: ParamsAble = "00111234",
     *,
     defaults=None,
     name=None,
@@ -176,8 +173,19 @@ def mk_func_from_params(
     >>> f = mk_func_from_params(params="00111234")
     >>> str(Sig(f))
     '(a00, a01, /, a12, a13, a14, *a25, a36, **a47)'
+
+    The next one is convoluted on purpose, to try to break the tools with name clashing,
+    using the name a47 several times.
+    The ``a47=...`` in the input could have been any name, but is bound to be caught in
+    the ``**a47`` variadic argument.
+    Therefore, the first ``a47=...`` of the output is saying "I'm the variadic that caught
+    that input". The ``{'a47': ...`` following it is saying "the user input an extra
+    keyword argument that they called `a47`. Then the ``{"a47": 7, "a47_": -7}`` is just
+    the value of that keyword argument.
+
     >>> f(0, 1, 2, 3, 4, 5, -5, a36=6, a47={"a47": 7, "a47_": -7})
-    "a00=0, a01=1, a12=2, a13=3, a14=4, a25=(5, -5), a36=6, a47={'a47': 7, 'a47_': -7}"
+    "a00=0, a01=1, a12=2, a13=3, a14=4, a25=(5, -5), a36=6, a47={'a47': {'a47': 7, 'a47_': -7}}"
+
     >>> f(0, 1, 2, a13=3, a14=4, a36=6)
     'a00=0, a01=1, a12=2, a13=3, a14=4, a25=(), a36=6, a47={}'
 
@@ -209,9 +217,7 @@ def mk_func_from_params(
 
     @sig
     def arg_str_func(*args, **kwargs):
-        _call_kwargs = sig.kwargs_from_args_and_kwargs(
-            args, kwargs, apply_defaults=True
-        )
+        _call_kwargs = sig.map_arguments(args, kwargs, apply_defaults=True)
         return callback(_call_kwargs)
 
     arg_str_func.__name__ = name or _params_to_name(params)
@@ -221,7 +227,7 @@ def mk_func_from_params(
 
 def _sig_to_str_of_call_args_code_str(sig: Sig):
     return (
-        'return ' + 'f"' + _str_of_call_args({p: f'{{{p}}}' for p in sig.names}) + '"'
+        "return " + 'f"' + _str_of_call_args({p: f"{{{p}}}" for p in sig.names}) + '"'
     )
 
 
@@ -229,8 +235,8 @@ def _is_simple_expression(code_lines):
     if len(code_lines) == 1:
         line = code_lines[0].strip()
         if not (
-            line in {'pass', '...'}
-            or line.startswith('return')
+            line in {"pass", "..."}
+            or line.startswith("return")
             or (line.startswith('"') and line.endswith("'"))
         ):
             return True
@@ -239,14 +245,14 @@ def _is_simple_expression(code_lines):
 
 
 def sig_to_func(
-    sig: ParamsAble = '00111234',
-    code_lines: Union[
-        str, Iterable, Callable[[Sig], str]
-    ] = _sig_to_str_of_call_args_code_str,
+    sig: ParamsAble = "00111234",
+    code_lines: (
+        str | Iterable | Callable[[Sig], str]
+    ) = _sig_to_str_of_call_args_code_str,
     *,
-    name: Optional[str] = None,
-    globals: Optional[dict] = None,
-    locals: Optional[Mapping] = None,
+    name: str | None = None,
+    globals: dict | None = None,
+    locals: Mapping | None = None,
 ):
     """
     Make a function from a signature
@@ -323,12 +329,12 @@ def sig_to_func(
     if callable(code_lines):
         code_lines = code_lines(sig)  # call the function on sig to get lines
     if isinstance(code_lines, str):
-        code_lines = code_lines.split('\n')
+        code_lines = code_lines.split("\n")
     if _is_simple_expression(code_lines):
         # If code_lines has only one line and it seems it's an expression, prepend return
-        code_lines = [f'return {code_lines[0]}']
-    code_string = '\n\t'.join(code_lines)
-    func_def_string = f'def {name}{sig}:\n\t{code_string}'
+        code_lines = [f"return {code_lines[0]}"]
+    code_string = "\n\t".join(code_lines)
+    func_def_string = f"def {name}{sig}:\n\t{code_string}"
     _locals = locals or {}
     exec(func_def_string, globals, _locals)
     return _locals[name]
@@ -371,24 +377,24 @@ def _args_kwargs_combinations(args, kwargs):
 def variadic_type(sig, variadics):
     var_kinds = [sig.kinds[param] for param in variadics]
     if not variadics:
-        return 'no_var'
+        return "no_var"
     if VP in var_kinds and VK not in var_kinds:
-        return 'vp_only'
+        return "vp_only"
     if VK in var_kinds and VP not in var_kinds:
-        return 'vk_only'
+        return "vk_only"
     else:
-        return 'vp_vk'
+        return "vp_vk"
 
 
 def create_variadic_source(sig, variadics, dflt_source):
     var_type = variadic_type(sig, variadics)
-    if var_type == 'no_var':
+    if var_type == "no_var":
         result = ((), {})
-    elif var_type == 'vp_only':
+    elif var_type == "vp_only":
         result = (dflt_source[0], {})
-    elif var_type == 'vk_only':
+    elif var_type == "vk_only":
         result = ((), dflt_source[1])
-    elif var_type == 'vp_vk':
+    elif var_type == "vp_vk":
         result = dflt_source
     return result
 
@@ -396,13 +402,13 @@ def create_variadic_source(sig, variadics, dflt_source):
 # TODO: Get rid of ignore_variadics once using code is refactored
 def sig_to_inputs(
     sig: SignatureAble,
-    argument_vals: Optional[Iterable] = None,
+    argument_vals: Iterable | None = None,
     *,
-    variadics_source: Tuple[tuple, dict] = (
-        ('args1', 'args2'),
-        {'kwargs1': 'kwargs1_val'},
+    variadics_source: tuple[tuple, dict] = (
+        ("args1", "args2"),
+        {"kwargs1": "kwargs1_val"},
     ),
-) -> Iterator[Tuple[tuple, dict]]:
+) -> Iterator[tuple[tuple, dict]]:
     """Generate all kind-valid (arg, kwargs) input combinations for a function with a
     given signature ``sig``, with argument values taken from the ``argument_vals``
 
@@ -467,7 +473,7 @@ def sig_to_inputs(
                     already_yielded.append(input_)
 
 
-def _sig_to_inputs(po=0, pk=0, ko=0, argument_vals: Optional[Iterable] = None):
+def _sig_to_inputs(po=0, pk=0, ko=0, argument_vals: Iterable | None = None):
     """
 
     >>> list(_sig_to_inputs(2,2,2))
@@ -560,15 +566,15 @@ from i2 import Pipe
 import re
 
 _signature_msg_patterns = [
-    'keyword arguments$',
-    'invalid keyword argument',
-    'expected at most',
-    'keyword argument',
-    'got some positional\-only arguments passed as keyword arguments',
-    'no signature found',
+    "keyword arguments$",
+    "invalid keyword argument",
+    "expected at most",
+    "keyword argument",
+    r"got some positional\-only arguments passed as keyword arguments",
+    "no signature found",
 ]
 
-_signature_msg_regex = re.compile('|'.join(map('({})'.format, _signature_msg_patterns)))
+_signature_msg_regex = re.compile("|".join(map("({})".format, _signature_msg_patterns)))
 is_signature_msg = Pipe(_signature_msg_regex.search, bool)
 
 
@@ -600,12 +606,13 @@ def on_error_return_none(func, /, *args, **kwargs):
 
 call_raises_signature_error = Pipe(call_and_return_error, _is_signature_error)
 
-call_raises_signature_error.__doc__ = '''
+call_raises_signature_error.__doc__ = """
 >>> call_raises_signature_error(lambda x, /, y: None, 1, y=2)
 False
 >>> call_raises_signature_error(lambda x, /, y: None, x=1, y=2)
 True
-'''
+"""
+
 
 # Yes, I too see that this can be made into yet another Pipe!
 def function_is_compatible_with_signature(func, sig):
@@ -711,9 +718,9 @@ def trace_call(func, local_vars, name=None):
     if name is None:
         name = func.__name__
     return (
-        f'{name}('
-        + ', '.join(f'{argname}={local_vars[argname]}' for argname in Sig(func).names)
-        + ')'
+        f"{name}("
+        + ", ".join(f"{argname}={local_vars[argname]}" for argname in Sig(func).names)
+        + ")"
     )
 
 
