@@ -1,33 +1,374 @@
 """Misc util objects"""
+
 from operator import attrgetter, itemgetter
 import inspect
 import re
 import os
 import itertools
 import sys
-from functools import partial
+from functools import partial, wraps, cached_property, partialmethod
 import types
+from types import SimpleNamespace
 from typing import (
-    Mapping,
-    Callable,
+    Tuple,
     Any,
-    MutableMapping,
     Union,
     Optional,
-    Iterable,
     TypeVar,
+    KT,
+    Literal,
 )
+from collections.abc import Mapping, Callable, MutableMapping, Iterable, Iterator
+from collections import namedtuple
 import contextlib
 import io
 
-T = TypeVar('T')
+T = TypeVar("T")
+
+
+def asis(x: T) -> T:
+    """the identity function: f(x) := x (takes only one argument, and returns it)"""
+    return x
+
+
+def return_false(*args, **kwargs):
+    """a function that returns True (takes any number of arguments)"""
+    return False
+
+
+def return_true(*args, **kwargs):
+    """a function that returns False (takes any number of arguments)"""
+    return False
+
+
+def return_none(*args, **kwargs) -> None:
+    return None
+
+
+from typing import Any, Optional
+from collections.abc import MutableMapping
+
+
+def name_of_obj(
+    o: object,
+    *,
+    base_name_of_obj: Callable = attrgetter("__name__"),
+    caught_exceptions: tuple = (AttributeError,),
+    default_factory: Callable = return_none,
+) -> str | None:
+    """
+    Tries to find the (or "a") name for an object, even if `__name__` doesn't exist.
+
+    >>> name_of_obj(map)
+    'map'
+    >>> name_of_obj([1, 2, 3])
+    'list'
+    >>> name_of_obj(print)
+    'print'
+    >>> name_of_obj(lambda x: x)
+    '<lambda>'
+    >>> from functools import partial
+    >>> name_of_obj(partial(print, sep=","))
+    'print'
+    >>> from functools import cached_property
+    >>> class A:
+    ...     @property
+    ...     def prop(self):
+    ...         return 1.0
+    ...     @cached_property
+    ...     def cached_prop(self):
+    ...         return 2.0
+    >>> name_of_obj(A.prop)
+    'prop'
+    >>> name_of_obj(A.cached_prop)
+    'cached_prop'
+
+    Note that ``name_of_obj`` uses the ``__name__`` attribute as its base way to get
+    a name. You can customize this behavior though.
+    For example, see that:
+
+    >>> from inspect import Signature
+    >>> name_of_obj(Signature.replace)
+    'replace'
+
+    If you want to get the fully qualified name of an object, you can do:
+
+    >>> alt = partial(name_of_obj, base_name_of_obj=attrgetter('__qualname__'))
+    >>> alt(Signature.replace)
+    'Signature.replace'
+
+    """
+    try:
+        return base_name_of_obj(o)
+    except caught_exceptions:
+        kwargs = dict(
+            base_name_of_obj=base_name_of_obj,
+            caught_exceptions=caught_exceptions,
+            default_factory=default_factory,
+        )
+        if isinstance(o, (cached_property, partial, partialmethod)) and hasattr(
+            o, "func"
+        ):
+            return name_of_obj(o.func, **kwargs)
+        elif isinstance(o, property) and hasattr(o, "fget"):
+            return name_of_obj(o.fget, **kwargs)
+        elif hasattr(o, "__class__"):
+            return name_of_obj(type(o), **kwargs)
+        elif hasattr(o, "fset"):
+            return name_of_obj(o.fset, **kwargs)
+        return default_factory(o)
+
+
+def register_object(
+    obj: Any = None,
+    name: str | None = None,
+    *,
+    registry: MutableMapping,
+):
+    """
+    Register an object (e.g. function, class) in the global registry.
+
+    The raw use is to define a registry Mapping and then call this function with the registry and the object to register.
+
+    >>> registry = {}
+    >>> def wet():
+    ...     pass
+    >>> register_object(wet, registry=registry)  # doctest: +ELLIPSIS
+    <function wet at 0x...>
+    >>> registry  # doctest: +ELLIPSIS
+    {'wet': <function wet at 0x...>}
+
+    >>> register_object(wet, name='custom_name', registry=registry)  # doctest: +ELLIPSIS
+    <function wet at 0x...>
+    >>> registry  # doctest: +ELLIPSIS
+    {'wet': <function wet at 0x...>, 'custom_name': <function wet at 0x...>}
+
+    The most common use of this function is to use it as a decorator with a fixed (but mutable!) registry:
+
+    >>> another_registry = {}
+    >>> register_to_another = register_object(registry=another_registry)
+    >>> @register_to_another
+    ... def dry():
+    ...     pass
+    >>> another_registry  # doctest: +ELLIPSIS
+    {'dry': <function dry at 0x...>}
+
+    >>> @register_to_another('DRY')
+    ... def foo():
+    ...     pass
+    >>> another_registry  # doctest: +ELLIPSIS
+    {'dry': <function dry at 0x...>, 'DRY': <function foo at 0x...>}
+    """
+    if obj is None:
+        # if obj is None, it means we are using the decorator form
+        # (this is the register_object() case)
+        return partial(register_object, name=name, registry=registry)
+    if isinstance(obj, str):
+        # if the obj is a string, it is the name of the object to register
+        # (this is the register_object(name) case)
+        name = obj
+        return partial(register_object, name=name, registry=registry)
+
+    if name is None:
+        name = name_of_obj(obj)
+
+    registry[name] = obj
+    return obj
+
+
+ExceptionTypes = Union[BaseException, tuple[BaseException]]
+Handler = Callable[[BaseException], None]
+Handlers = Union[Handler, Mapping[KT, Handler]]
+
+ignore_exception = asis  # an alias of asis, to make it clear in context where used
+
+
+# Note: Similar functionality in meshed.slabs
+#       (https://github.com/i2mint/meshed/blob/61a5633cc8e1d4b4b26f31d3bf70d744aab327c4/meshed/slabs.py#L145)
+class ConditionalExceptionCatcher:
+    """
+    Context manager to catch exceptions of a certain type and instance condition.
+
+    :param exception_types: The type of exception to catch. Can be a single exception
+        type or a tuple of exception types.
+    :param exception_condition: A function that takes an exception instance and returns
+        a "key" value indicating whether the exception should be caught.
+        If the bool(key) is True, the exception is caught.
+        If the bool(key) is False, the exception is not caught.
+        The key can further be used to determine the handler to use, when the handlers
+        argument is a mapping.
+        The default is to catch all exceptions of the specified type(s).
+    :param handlers: Specification of how to handle the exceptions. Can be a single
+        function to run on the exception object when an exception of the specified
+        type is caught, or a mapping (e.g. dict) of handler functions, keyed by the
+        key returned by the exception_condition function.
+    :param prevent_propagation: Whether to prevent the exception from propagating. Defaults
+        to ``True``.
+
+    Example:
+
+    >>> exception_catcher = ConditionalExceptionCatcher(
+    ...     ValueError, lambda e: e.args[0] == 'foo', handlers=print
+    ... )
+    >>> with exception_catcher:
+    ...     raise ValueError('foo')
+    foo
+    >>> with exception_catcher:
+    ...     raise TypeError('foo')
+    Traceback (most recent call last):
+        ...
+    TypeError: foo
+    >>> with exception_catcher:
+    ...     raise ValueError('bar')
+    Traceback (most recent call last):
+        ...
+    ValueError: bar
+
+    """
+
+    def __init__(
+        self,
+        exception_types: ExceptionTypes,
+        exception_condition: Callable[[BaseException], bool] = return_true,
+        handlers: Callable[[BaseException], None] = ignore_exception,
+        *,
+        prevent_propagation=True,
+    ):
+        self.exception_types = exception_types
+        self.exception_condition = exception_condition
+        self.handlers = handlers
+        self.prevent_propagation = prevent_propagation
+
+        assert callable(
+            self.exception_condition
+        ), f"Expected a callable for {self.exception_condition=}"
+        assert callable(self.handlers) or isinstance(
+            self.handlers, Mapping
+        ), f"Expected a callable or a mapping for {self.handlers=}"
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        # TODO: Could extend exception_condition and/or handler to use traceback
+        if exc_type is self.exception_types and (
+            key := self.exception_condition(exc_value)
+        ):
+            if isinstance(self.handlers, Mapping):
+                handler = self.handlers[key]
+            else:
+                handler = self.handlers
+            handler(exc_value)
+            return self.prevent_propagation
+        return False  # Allows other exceptions to propagate
+
+
+# -------------------------------------------------------------------------------------
+# Attribute Mapping Classes
+# (Vendored in dol)
+
+
+class AttributeMapping(SimpleNamespace, Mapping[str, Any]):
+    """
+    A read-only mapping with attribute access.
+
+    Useful when you want mapping interface but don't need mutation.
+
+    Examples:
+
+    >>> ns = AttributeMapping(x=10, y=20)
+    >>> ns.x
+    10
+    >>> ns['y']
+    20
+    >>> list(ns)
+    ['x', 'y']
+    """
+
+    @classmethod
+    def from_mapping(self, mapping: Mapping[str, Any]) -> "AttributeMapping":
+        """
+        Create an AttributeMapping from a regular mapping.
+
+        This is useful when you want to convert a dictionary or other mapping
+        into an AttributeMapping for attribute-style access.
+        """
+        return self(**mapping)
+
+    def __getitem__(self, key: str) -> Any:
+        """Get item with proper KeyError on missing keys."""
+        return _get_attr_or_key_error(self, key)
+
+    def __iter__(self) -> Iterator[str]:
+        """Iterate over attribute names."""
+        return iter(self.__dict__)
+
+    def __len__(self) -> int:
+        """Return number of attributes."""
+        return len(self.__dict__)
+
+
+class AttributeMutableMapping(AttributeMapping, MutableMapping[str, Any]):
+    """
+    A mutable mapping that provides both attribute and dictionary-style access.
+
+    Extends AttributeMapping with mutation capabilities,
+    ensuring proper error handling and protocol compliance.
+
+    Examples:
+
+    >>> ns = AttributeMutableMapping(apple=1, banana=2)
+    >>> ns.apple
+    1
+    >>> ns['banana']
+    2
+    >>> ns['cherry'] = 3
+    >>> ns.cherry
+    3
+    >>> list(ns)
+    ['apple', 'banana', 'cherry']
+    >>> len(ns)
+    3
+    >>> 'apple' in ns
+    True
+    >>> del ns['banana']
+    >>> 'banana' in ns
+    False
+    """
+
+    def __setitem__(self, key: str, value: Any) -> None:
+        """Set item via attribute assignment."""
+        setattr(self, key, value)
+
+    def __delitem__(self, key: str) -> None:
+        """Delete item with proper KeyError on missing keys."""
+        try:
+            delattr(self, key)
+        except AttributeError:
+            raise KeyError(key)
+
+
+def _get_attr_or_key_error(obj: object, key: str) -> Any:
+    """
+    Get attribute or raise KeyError if not found.
+
+    Helper function to maintain consistent error handling across
+    mapping implementations.
+    """
+    try:
+        return getattr(obj, key)
+    except AttributeError:
+        raise KeyError(key)
+
+
+# -------------------------------------------------------------------------------------
 
 
 @contextlib.contextmanager
-def FileLikeObject(file, *, io_cls=io.BytesIO, open_mode='rb'):
+def FileLikeObject(file, *, io_cls=io.BytesIO, open_mode="rb"):
     """Context manager for file-like objects.
 
-    The purpose of this context manager is to be able to ensure we have a file-like 
+    The purpose of this context manager is to be able to ensure we have a file-like
     object interface to work with, regardless of whether we are given a file path,
     bytes of a file, or an open file pointer.
 
@@ -95,18 +436,18 @@ def copy_func(
     """
     from types import FunctionType
 
-    code = code or getattr(func, '__code__', None)
+    code = code or getattr(func, "__code__", None)
     if not isinstance(code, types.CodeType):
-        raise TypeError(f'Expected a types.CodeType object, but got {type(code)=}')
-    globals_ = globals_ or getattr(func, '__globals__', {})
+        raise TypeError(f"Expected a types.CodeType object, but got {type(code)=}")
+    globals_ = globals_ or getattr(func, "__globals__", {})
     new_func = FunctionType(
         code,  # if your func doesn't have a __code__ attr, can't make a copy!
         globals_,
-        name=getattr(func, '__name__', None),
-        argdefs=getattr(func, '__defaults__', None),
-        closure=getattr(func, '__closure__', None),
+        name=getattr(func, "__name__", None),
+        argdefs=getattr(func, "__defaults__", None),
+        closure=getattr(func, "__closure__", None),
     )
-    if hasattr(func, '__kwdefaults__'):
+    if hasattr(func, "__kwdefaults__"):
         new_func.__kwdefaults__ = func.__kwdefaults__
     if copy_dict:
         new_func.__dict__.update(func.__dict__)
@@ -118,7 +459,7 @@ class OverwritesForbidden(ValueError):
 
 
 def is_lambda(func):
-    return getattr(func, '__name__', None) == '<lambda>'
+    return getattr(func, "__name__", None) == "<lambda>"
 
 
 # TODO: Fragile. Make more robust.
@@ -129,7 +470,7 @@ def lambda_code(lambda_func) -> str:
 
     """
     func_str = str(inspect.getsourcelines(lambda_func)[0])
-    return func_str.strip("['\\n']").split(' = ')[1]
+    return func_str.strip("['\\n']").split(" = ")[1]
 
 
 # TODO: Only works with lambdas so either assert function is a lambda on construction
@@ -153,7 +494,7 @@ class PicklableLambda:
     def __init__(self, func, name=None):
         self.func = func
         self.__signature__ = inspect.signature(self.func)
-        self.__name__ = name or getattr(func, '__name__', type(self).__name__)
+        self.__name__ = name or getattr(func, "__name__", type(self).__name__)
 
     def __getstate__(self):
         return lambda_code(self.func), self.__name__
@@ -167,7 +508,7 @@ class PicklableLambda:
         return self.func(*args, **kwargs)
 
     def __repr__(self):
-        return f'<{type(self).__name__}({self.__name__})>'
+        return f"<{type(self).__name__}({self.__name__})>"
 
 
 def ensure_identifiers(
@@ -277,7 +618,7 @@ def insert_name_based_objects_in_scope(
             scope[name] = factory(name)
         else:
             raise OverwritesForbidden(
-                f'This key already exisited and is not allowed to be overwritten'
+                f"This key already exisited and is not allowed to be overwritten"
             )
 
 
@@ -321,7 +662,7 @@ def dflt_idx_preprocessor(obj, idx):
         raise KeyError(f"Couldn't extract a {idx} from object {obj}")
 
 
-def path_extractor(tree, path, getter=dflt_idx_preprocessor, *, path_sep='.'):
+def path_extractor(tree, path, getter=dflt_idx_preprocessor, *, path_sep="."):
     """Get items from a tree-structured object from a sequence of tree-traversal indices.
 
     :param tree: The object you want to extract values from:
@@ -380,7 +721,7 @@ def path_extractor(tree, path, getter=dflt_idx_preprocessor, *, path_sep='.'):
         return tree
     else:
         idx, *path = path  # extract path[0] as idx & update path to path[1:]
-        if isinstance(idx, str) and idx == '*':
+        if isinstance(idx, str) and idx == "*":
             idx = lambda x: True  # use a filter function (but filter everything in)
         if callable(idx) and not isinstance(idx, LiteralVal):
             # If idx is a non-literal callable, consider it as a filter to be applied
@@ -411,32 +752,6 @@ def dp_get(d, dot_path):
     return path_extractor(d, dot_path, lambda d, k: d[k])
 
 
-def get_app_data_folder():
-    """
-    Returns the full path of a directory suitable for storing application-specific data.
-
-    On Windows, this is typically %APPDATA%.
-    On macOS, this is typically ~/.config.
-    On Linux, this is typically ~/.config.
-
-    Returns:
-        str: The full path of the app data folder.
-
-    See https://github.com/i2mint/i2mint/issues/1.
-    """
-    if os.name == 'nt':
-        # Windows
-        app_data_folder = os.getenv('APPDATA')
-    elif os.name == 'darwin':
-        # macOS
-        app_data_folder = os.path.expanduser('~/.config')
-    else:
-        # Linux/Unix
-        app_data_folder = os.path.expanduser('~/.config')
-
-    return app_data_folder
-
-
 class lazyprop:
     """
     A descriptor implementation of lazyprop (cached property) from David Beazley's "Python Cookbook" book.
@@ -499,7 +814,7 @@ class FrozenDict(dict):
 
     """
 
-    __slots__ = ('_hash',)
+    __slots__ = ("_hash",)
 
     def updated(self, *a, **kw):
         """Make a copy and add items from a dictionary or iterable (and/or
@@ -517,7 +832,7 @@ class FrozenDict(dict):
 
     def __repr__(self):
         cn = self.__class__.__name__
-        return '%s(%s)' % (cn, dict.__repr__(self))
+        return "{}({})".format(cn, dict.__repr__(self))
 
     def __reduce_ex__(self, protocol):
         return type(self), (dict(self),)
@@ -541,8 +856,8 @@ class FrozenDict(dict):
 
     # block everything else
     def _raise_frozen_typeerror(self, *a, **kw):
-        'raises a TypeError, because FrozenDicts are immutable'
-        raise TypeError('%s object is immutable' % self.__class__.__name__)
+        "raises a TypeError, because FrozenDicts are immutable"
+        raise TypeError("%s object is immutable" % self.__class__.__name__)
 
     __setitem__ = __delitem__ = update = _raise_frozen_typeerror
     setdefault = pop = popitem = clear = _raise_frozen_typeerror
@@ -560,9 +875,9 @@ function_type = type(
 )  # using this instead of callable() because classes are callable, for instance
 
 
-class NoDefault(object):
+class NoDefault:
     def __repr__(self):
-        return 'no_default'
+        return "no_default"
 
 
 no_default = NoDefault()
@@ -573,7 +888,7 @@ class imdict(dict):
         return id(self)
 
     def _immutable(self, *args, **kws):
-        raise TypeError('object is immutable')
+        raise TypeError("object is immutable")
 
     __setitem__ = _immutable
     __delitem__ = _immutable
@@ -586,6 +901,8 @@ class imdict(dict):
 
 def inject_method(self, method_function, method_name=None):
     """
+    Inject a method into an object instance.
+
     method_function could be:
         * a function
         * a {method_name: function, ...} dict (for multiple injections)
@@ -613,21 +930,29 @@ def inject_method(self, method_function, method_name=None):
 
 
 def get_function_body(func):
+    """
+    Get the body of a function as a string.
+
+    :param func: The function to get the body of.
+
+    :return: The body of the function as a string.
+
+    """
     source_lines = inspect.getsourcelines(func)[0]
-    source_lines = itertools.dropwhile(lambda x: x.startswith('@'), source_lines)
+    source_lines = itertools.dropwhile(lambda x: x.startswith("@"), source_lines)
     line = next(source_lines).strip()
-    if not line.startswith('def ') and not line.startswith('class'):
-        return line.rsplit(':')[-1].strip()
-    elif not line.endswith(':'):
+    if not line.startswith("def ") and not line.startswith("class"):
+        return line.rsplit(":")[-1].strip()
+    elif not line.endswith(":"):
         for line in source_lines:
             line = line.strip()
-            if line.endswith(':'):
+            if line.endswith(":"):
                 break
     # Handle functions that are not one-liners
     first_line = next(source_lines)
     # Find the indentation of the first line
     indentation = len(first_line) - len(first_line.lstrip())
-    return ''.join(
+    return "".join(
         [first_line[indentation:]] + [line[indentation:] for line in source_lines]
     )
 
@@ -641,15 +966,15 @@ class MissingArgument(ValueError):
 
 
 def _default_sentinel_repr_method(self):
-    return '%s(%r)' % (self.__class__.__name__, self.__name__)
+    return "{}({!r})".format(self.__class__.__name__, self.__name__)
 
 
 def mk_sentinel(
     name,
     boolean_value: bool = False,
-    repr_: Union[str, Callable] = _default_sentinel_repr_method,
+    repr_: str | Callable = _default_sentinel_repr_method,
     *,
-    module: Optional[str] = None,
+    module: str | None = None,
 ):
     """Creates and returns a new **instance** of a new class, suitable for usage as a
     "sentinel" since it is a kind of singleton (there can be only one instance of it.)
@@ -764,7 +1089,7 @@ def mk_sentinel(
 
     """
 
-    class Sentinel(object):
+    class Sentinel:
         def __init__(self):
             self.__name__ = name
 
@@ -785,26 +1110,26 @@ def mk_sentinel(
         # TODO: Try to use something else than hidden _getframe
         # TODO: extract this module resolver so can be reused (_getframe(2)?)
         frame = sys._getframe(1)
-        module = frame.f_globals.get('__name__')
+        module = frame.f_globals.get("__name__")
 
     if not module or module not in sys.modules:
         raise ValueError(
-            'Pickleable sentinel objects can only be made from top-level module scopes'
+            "Pickleable sentinel objects can only be made from top-level module scopes"
         )
     Sentinel.__module__ = module
 
     return Sentinel()
 
 
-def _indent(text, margin, newline='\n', key=bool):
-    'based on boltons.strutils.indent'
+def _indent(text, margin, newline="\n", key=bool):
+    "based on boltons.strutils.indent"
     indented_lines = [
         (margin + line if key(line) else line) for line in text.splitlines()
     ]
     return newline.join(indented_lines)
 
 
-NO_DEFAULT = mk_sentinel('NO_DEFAULT', boolean_value=False)
+NO_DEFAULT = mk_sentinel("NO_DEFAULT", boolean_value=False)
 
 # --------------------------------------------------------------------------------------
 # FunctionBuilder, vendored and adapted from boltons.funcutils
@@ -821,10 +1146,10 @@ def inspect_formatargspec(
     kwonlydefaults={},
     annotations={},
     formatarg=str,
-    formatvarargs=lambda name: '*' + name,
-    formatvarkw=lambda name: '**' + name,
-    formatvalue=lambda value: '=' + repr(value),
-    formatreturns=lambda text: ' -> ' + text,
+    formatvarargs=lambda name: "*" + name,
+    formatvarkw=lambda name: "**" + name,
+    formatvalue=lambda value: "=" + repr(value),
+    formatreturns=lambda text: " -> " + text,
     formatannotation=formatannotation,
 ):
     """Copy formatargspec from python 3.7 standard library.
@@ -838,7 +1163,7 @@ def inspect_formatargspec(
     def formatargandannotation(arg):
         result = formatarg(arg)
         if arg in annotations:
-            result += ': ' + formatannotation(annotations[arg])
+            result += ": " + formatannotation(annotations[arg])
         return result
 
     specs = []
@@ -853,7 +1178,7 @@ def inspect_formatargspec(
         specs.append(formatvarargs(formatargandannotation(varargs)))
     else:
         if kwonlyargs:
-            specs.append('*')
+            specs.append("*")
     if kwonlyargs:
         for kwonlyarg in kwonlyargs:
             spec = formatargandannotation(kwonlyarg)
@@ -862,13 +1187,13 @@ def inspect_formatargspec(
             specs.append(spec)
     if varkw is not None:
         specs.append(formatvarkw(formatargandannotation(varkw)))
-    result = '(' + ', '.join(specs) + ')'
-    if 'return' in annotations:
-        result += formatreturns(formatannotation(annotations['return']))
+    result = "(" + ", ".join(specs) + ")"
+    if "return" in annotations:
+        result += formatreturns(formatannotation(annotations["return"]))
     return result
 
 
-class FunctionBuilder(object):
+class FunctionBuilder:
     """The FunctionBuilder type provides an interface for programmatically
     creating new functions, either based on existing functions or from
     scratch.
@@ -935,29 +1260,29 @@ class FunctionBuilder(object):
     """
 
     _argspec_defaults = {
-        'args': list,
-        'varargs': lambda: None,
-        'varkw': lambda: None,
-        'defaults': lambda: None,
-        'kwonlyargs': list,
-        'kwonlydefaults': dict,
-        'annotations': dict,
+        "args": list,
+        "varargs": lambda: None,
+        "varkw": lambda: None,
+        "defaults": lambda: None,
+        "kwonlyargs": list,
+        "kwonlydefaults": dict,
+        "annotations": dict,
     }
 
     @classmethod
     def _argspec_to_dict(cls, f):
         argspec = inspect.getfullargspec(f)
-        return dict((attr, getattr(argspec, attr)) for attr in cls._argspec_defaults)
+        return {attr: getattr(argspec, attr) for attr in cls._argspec_defaults}
 
     _defaults = {
-        'doc': str,
-        'dict': dict,
-        'is_async': lambda: False,
-        'module': lambda: None,
-        'body': lambda: 'pass',
-        'indent': lambda: 4,
-        'annotations': dict,
-        'filename': lambda: 'boltons.funcutils.FunctionBuilder',
+        "doc": str,
+        "dict": dict,
+        "is_async": lambda: False,
+        "module": lambda: None,
+        "body": lambda: "pass",
+        "indent": lambda: 4,
+        "annotations": dict,
+        "filename": lambda: "boltons.funcutils.FunctionBuilder",
     }
 
     _defaults.update(_argspec_defaults)
@@ -973,7 +1298,7 @@ class FunctionBuilder(object):
             setattr(self, a, val)
 
         if kw:
-            raise TypeError('unexpected kwargs: %r' % kw.keys())
+            raise TypeError("unexpected kwargs: %r" % kw.keys())
         return
 
     # def get_argspec(self):  # TODO
@@ -994,12 +1319,12 @@ class FunctionBuilder(object):
         )
 
     _KWONLY_MARKER = re.compile(
-        r'''
+        r"""
     \*     # a star
     \s*    # followed by any amount of whitespace
     ,      # followed by a comma
     \s*    # followed by any amount of whitespace
-    ''',
+    """,
         re.VERBOSE,
     )
 
@@ -1007,8 +1332,8 @@ class FunctionBuilder(object):
         kwonly_pairs = None
         formatters = {}
         if self.kwonlyargs:
-            kwonly_pairs = dict((arg, arg) for arg in self.kwonlyargs)
-            formatters['formatvalue'] = lambda value: '=' + value
+            kwonly_pairs = {arg: arg for arg in self.kwonlyargs}
+            formatters["formatvalue"] = lambda value: "=" + value
 
         sig = inspect_formatargspec(
             self.args,
@@ -1020,7 +1345,7 @@ class FunctionBuilder(object):
             {},
             **formatters,
         )
-        sig = self._KWONLY_MARKER.sub('', sig)
+        sig = self._KWONLY_MARKER.sub("", sig)
         return sig[1:-1]
 
     @classmethod
@@ -1032,21 +1357,21 @@ class FunctionBuilder(object):
         # TODO: copy_body? gonna need a good signature regex.
         # TODO: might worry about __closure__?
         if not callable(func):
-            raise TypeError('expected callable object, not %r' % (func,))
+            raise TypeError("expected callable object, not {!r}".format(func))
 
         if isinstance(func, partial):
             kwargs = {
-                'name': func.__name__,
-                'doc': func.__doc__,
-                'module': getattr(func, '__module__', None),  # e.g., method_descriptor
-                'annotations': getattr(func, '__annotations__', {}),
-                'dict': getattr(func, '__dict__', {}),
+                "name": func.__name__,
+                "doc": func.__doc__,
+                "module": getattr(func, "__module__", None),  # e.g., method_descriptor
+                "annotations": getattr(func, "__annotations__", {}),
+                "dict": getattr(func, "__dict__", {}),
             }
 
         kwargs.update(cls._argspec_to_dict(func))
 
         if inspect.iscoroutinefunction(func):
-            kwargs['is_async'] = True
+            kwargs["is_async"] = True
 
         return cls(**kwargs)
 
@@ -1070,15 +1395,15 @@ class FunctionBuilder(object):
         execdict = execdict or {}
         body = self.body or self._default_body
 
-        tmpl = 'def {name}{sig_str}:'
-        tmpl += '\n{body}'
+        tmpl = "def {name}{sig_str}:"
+        tmpl += "\n{body}"
 
         if self.is_async:
-            tmpl = 'async ' + tmpl
+            tmpl = "async " + tmpl
 
-        body = _indent(self.body, ' ' * self.indent)
+        body = _indent(self.body, " " * self.indent)
 
-        name = self.name.replace('<', '_').replace('>', '_')  # lambdas
+        name = self.name.replace("<", "_").replace(">", "_")  # lambdas
         src = tmpl.format(
             name=name,
             sig_str=self.get_sig_str(with_annotations=False),
@@ -1111,13 +1436,13 @@ class FunctionBuilder(object):
         ret = dict(
             reversed(list(zip(reversed(self.args), reversed(self.defaults or []))))
         )
-        kwonlydefaults = getattr(self, 'kwonlydefaults', None)
+        kwonlydefaults = getattr(self, "kwonlydefaults", None)
         if kwonlydefaults:
             ret.update(kwonlydefaults)
         return ret
 
     def get_arg_names(self, only_required=False):
-        arg_names = tuple(self.args) + tuple(getattr(self, 'kwonlyargs', ()))
+        arg_names = tuple(self.args) + tuple(getattr(self, "kwonlyargs", ()))
         if only_required:
             defaults_dict = self.get_defaults_dict()
             arg_names = tuple([an for an in arg_names if an not in defaults_dict])
@@ -1130,11 +1455,13 @@ class FunctionBuilder(object):
         """
         if arg_name in self.args:
             raise ExistingArgument(
-                'arg %r already in func %s arg list' % (arg_name, self.name)
+                "arg {!r} already in func {} arg list".format(arg_name, self.name)
             )
         if arg_name in self.kwonlyargs:
             raise ExistingArgument(
-                'arg %r already in func %s kwonly arg list' % (arg_name, self.name)
+                "arg {!r} already in func {} kwonly arg list".format(
+                    arg_name, self.name
+                )
             )
         if not kwonly:
             self.args.append(arg_name)
@@ -1167,8 +1494,8 @@ class FunctionBuilder(object):
             except (AttributeError, ValueError):
                 # py2, or py3 and missing from both
                 exc = MissingArgument(
-                    'arg %r not found in %s argument list:'
-                    ' %r' % (arg_name, self.name, args)
+                    "arg %r not found in %s argument list:"
+                    " %r" % (arg_name, self.name, args)
                 )
                 exc.arg_name = arg_name
                 raise exc
@@ -1180,10 +1507,88 @@ class FunctionBuilder(object):
         return
 
     def _compile(self, src, execdict):
-        filename = '<%s-%d>' % (self.filename, next(self._compile_count),)
+        filename = "<%s-%d>" % (
+            self.filename,
+            next(self._compile_count),
+        )
         try:
-            code = compile(src, filename, 'single')
+            code = compile(src, filename, "single")
             exec(code, execdict)
         except Exception:
             raise
         return execdict
+
+
+def deprecation_of(func, old_name):
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        from warnings import warn
+
+        warn(
+            f"`{old_name}` is deprecated. Use `{func.__module__}.{func.__qualname__}` instead.",
+            DeprecationWarning,
+        )
+        return func(*args, **kwargs)
+
+    return wrapper
+
+
+# ----------------------------------------------------------------------------------------------------------------------
+# More or less vendored from config2py
+
+FolderSpec = namedtuple("FolderSpec", ["env_var", "default_path"])
+
+if os.name == "nt":
+    APP_FOLDER_STANDARDS = dict(
+        config=FolderSpec("APPDATA", os.getenv("APPDATA", "")),
+        data=FolderSpec("LOCALAPPDATA", os.getenv("LOCALAPPDATA", "")),
+        cache=FolderSpec(
+            "LOCALAPPDATA", os.path.join(os.getenv("LOCALAPPDATA", ""), "Temp")
+        ),
+        state=FolderSpec("LOCALAPPDATA", os.getenv("LOCALAPPDATA", "")),
+        runtime=FolderSpec("TEMP", os.getenv("TEMP", "")),
+    )
+else:
+    APP_FOLDER_STANDARDS = dict(
+        config=FolderSpec("XDG_CONFIG_HOME", "~/.config"),
+        data=FolderSpec("XDG_DATA_HOME", "~/.local/share"),
+        cache=FolderSpec("XDG_CACHE_HOME", "~/.cache"),
+        state=FolderSpec("XDG_STATE_HOME", "~/.local/state"),
+        runtime=FolderSpec("XDG_RUNTIME_DIR", "/tmp"),
+    )
+
+AppFolderKind = Literal["config", "data", "cache", "state", "runtime"]
+DFLT_APP_FOLDER_KIND = "config"
+
+
+def get_app_folder(folder_kind: AppFolderKind = DFLT_APP_FOLDER_KIND):
+    """
+    Get the full path of a directory suitable for storing application-specific configs,
+    (or data, or cache, or state or runtime)
+
+    On Windows, this is typically %APPDATA%.
+    On macOS, this is typically ~/.config.
+    On Linux, this is typically ~/.config.
+
+    Parameters:
+        folder_kind (str): The kind of folder to get. One of 'config', 'data', 'cache', 'state', 'runtime'.
+            Defaults to 'config'.
+            Here are concise explanations for each folder kind:
+            **config**: User preferences and settings files (e.g., API keys, theme preferences, editor settings). Files users might edit manually or that define how the app behaves.
+            **data**: Essential user-created content and application state (e.g., databases, saved games, user documents, session files). Data that should be backed up and persists across updates.
+            **cache**: Temporary, regeneratable files (e.g., downloaded images, compiled assets, web cache). Can be safely deleted to free space without losing user work.
+            **state**: Application state and logs that persist between sessions but aren't critical user data (e.g., command history, undo history, recently opened files, log files). Unlike cache, shouldn't be auto-deleted.
+            **runtime**: Temporary runtime files that only exist while the app runs (e.g., PID files, Unix sockets, lock files, named pipes). Typically cleared on logout/reboot.
+            **TL;DR**: config = settings, data = user files, cache = disposable, state = logs/history, runtime = process files.
+
+    Returns:
+        str: The full path of the app data folder.
+
+    See https://github.com/i2mint/i2mint/issues/1.
+    """
+    env_var, default = APP_FOLDER_STANDARDS[folder_kind]
+    return os.path.expanduser(os.getenv(env_var, default))
+
+
+get_app_config_folder = partial(get_app_folder, folder_kind="config")
+get_app_data_folder = partial(get_app_folder, folder_kind="data")
