@@ -66,17 +66,14 @@ How the ``Ingress`` class (ingress templated function maker) works:
 from functools import wraps, partial
 from inspect import Parameter, signature
 from typing import (
-    Mapping,
-    Callable,
     Any,
     Optional,
     Union,
-    Iterable,
-    Sequence,
     NewType,
     Dict,
     Tuple,
 )
+from collections.abc import Mapping, Callable, Iterable, Sequence
 
 from types import MethodType
 from inspect import Parameter
@@ -91,10 +88,12 @@ from i2.signatures import (
     parameter_to_dict,
     call_forgivingly,
     _call_forgivingly,
+    kind_forgiving_func,
 )
 from i2.errors import OverwritesNotAllowed
 from i2.multi_object import Pipe
 from i2.deco import double_up_as_factory
+from i2.util import deprecation_of
 
 # ---------------------------------------------------------------------------------------
 # Wrap
@@ -152,7 +151,7 @@ class _Wrap:
         # have had. It should be before anything else so it doesn't overwrite stuff
         # that we may add to self in init (like .func for example!)
         wraps(func)(self)  # TODO: should we really copy everything by default?
-        if (name := kwargs.get('name', None)) is not None:
+        if (name := kwargs.get("name", None)) is not None:
             self.__name__ = name
         self.func = func  # Note: overwrites self.func that wraps MAY have inserted
         self.__wrapped__ = func
@@ -175,8 +174,8 @@ class _Wrap:
 
     def __repr__(self):
         # TODO: Replace i2.Wrap with dynamic (Wrap or Wrapx)
-        name = getattr(self, '__name__', None) or 'Wrap'
-        return f'<i2.Wrap {name}{signature(self)}>'
+        name = getattr(self, "__name__", None) or "Wrap"
+        return f"<i2.Wrap {name}{signature(self)}>"
 
     # TODO: Don't know exactly what I'm doing below. Review with someone!
     def __set_name__(self, owner, name):
@@ -317,12 +316,11 @@ class Wrap(_Wrap):
         else:
 
             if isinstance(ingress, MakeFromFunc):
-                self.ingress = ingress  # the ingress function is
                 func_to_ingress = ingress  # it's not the ingress function itself
                 # ... but an ingress factory: Should make the ingress in function of func
                 self.ingress = func_to_ingress(func)
             else:
-                assert callable(ingress), f'Should be callable: {ingress}'
+                assert callable(ingress), f"Should be callable: {ingress}"
                 self.ingress = ingress
 
             ingress_sig = Sig(self.ingress)
@@ -481,7 +479,7 @@ def append_empty_args(func):
 
 
 class Ingress:
-    """The Ingress class offers a template for creating ingress classes.
+    r"""The Ingress class offers a template for creating ingress classes.
 
     Note that when writing a decorator with ``i2.wrapper``, you're usually better off
     writing an ingress function for the purpose. As a result, your code will usually
@@ -611,7 +609,7 @@ class Ingress:
     def __init__(
         self,
         inner_sig,
-        kwargs_trans: Optional[KwargsTrans] = None,
+        kwargs_trans: KwargsTrans | None = None,
         outer_sig=None,
         *,
         allow_excess=True,
@@ -642,8 +640,8 @@ class Ingress:
         ``inner_sig``, ``kwargs_trans`` and ``outer_sig`` are aligned.
 
         Namely, ``kwargs_trans`` must be able to handle outputs of
-        ``outer_sig.kwargs_from_args_and_kwargs`` and itself output kwargs that
-        can be handled by ``inner_sig.args_and_kwargs_from_kwargs``.
+        ``outer_sig.map_arguments`` and itself output kwargs that
+        can be handled by ``inner_sig.mk_args_and_kwargs``.
 
         """
         self.inner_sig = Sig(inner_sig)
@@ -666,7 +664,7 @@ class Ingress:
 
     def __call__(self, *ingress_args, **ingress_kwargs):
         # Get the all-keywords version of the arguments (args,kwargs->kwargs)
-        func_kwargs = self.outer_sig.kwargs_from_args_and_kwargs(
+        func_kwargs = self.outer_sig.map_arguments(
             ingress_args, ingress_kwargs, apply_defaults=self.apply_defaults
         )
 
@@ -678,7 +676,7 @@ class Ingress:
         # Return an (args, kwargs) pair that respects the inner function's
         # argument kind restrictions.
         # TODO: Reflect on pros/cons of allow_excess=True
-        return self.inner_sig.args_and_kwargs_from_kwargs(
+        return self.inner_sig.mk_args_and_kwargs(
             func_kwargs,
             apply_defaults=True,
             allow_excess=self.allow_excess,
@@ -686,7 +684,7 @@ class Ingress:
         )
 
     def __repr__(self):
-        return f'Ingress signature: {signature(self)}'
+        return f"Ingress signature: {signature(self)}"
 
     def wrap(self, func: Callable, egress=None, *, name=None) -> Wrap:
         """Convenience method to wrap a function with the instance ingress.
@@ -711,7 +709,7 @@ class Ingress:
         new_to_old_name = {v: k for k, v in old_to_new_name.items()}
         assert len(new_to_old_name) == len(
             old_to_new_name
-        ), f'Inversion is not possible since {old_to_new_name=} has duplicate values.'
+        ), f"Inversion is not possible since {old_to_new_name=} has duplicate values."
         return cls(
             wrapped,
             partial(Pipe(items_with_mapped_keys, dict), key_mapper=new_to_old_name),
@@ -758,7 +756,7 @@ def invert_map(d: dict):
     if len(new_d) == len(d):
         return new_d
     else:
-        raise ValueError(f'There are duplicate keys so I can invert map: {d}')
+        raise ValueError(f"There are duplicate keys so I can invert map: {d}")
 
 
 def parameters_to_dict(parameters):
@@ -767,7 +765,7 @@ def parameters_to_dict(parameters):
 
 def _handle_ingress_class_inputs(
     inner_sig,
-    kwargs_trans: Optional[KwargsTrans],
+    kwargs_trans: KwargsTrans | None,
     outer_sig,
     *,
     _allow_reordering=False,
@@ -902,7 +900,7 @@ class InnerMapIngress:
     def __init__(
         self,
         inner_sig,
-        kwargs_trans: Optional[KwargsTrans] = None,
+        kwargs_trans: KwargsTrans | None = None,
         *,
         _allow_reordering=False,
         **changes_for_name,
@@ -920,16 +918,16 @@ class InnerMapIngress:
         self.kwargs_trans = kwargs_trans
 
         outer_name_for_inner_name = {
-            inner_name: change['name']
+            inner_name: change["name"]
             for inner_name, change in changes_for_name.items()
-            if 'name' in change
+            if "name" in change
         }
         self.inner_name_for_outer_name = invert_map(outer_name_for_inner_name)
         self.outer_sig(self)
 
     def __call__(self, *ingress_args, **ingress_kwargs):
         # Get the all-keywords version of the arguments (args,kwargs->kwargs)
-        func_kwargs = self.outer_sig.kwargs_from_args_and_kwargs(
+        func_kwargs = self.outer_sig.map_arguments(
             ingress_args, ingress_kwargs, apply_defaults=True
         )
 
@@ -946,7 +944,7 @@ class InnerMapIngress:
         # TODO: See Ingress: Has allow_partial etc.
         # Return an (args, kwargs) pair the respects the inner function's
         # argument kind restrictions.
-        return self.inner_sig.args_and_kwargs_from_kwargs(func_kwargs)
+        return self.inner_sig.mk_args_and_kwargs(func_kwargs)
 
     # TODO: Use/test annotations of outer_sig
     @classmethod
@@ -1012,9 +1010,7 @@ class ArgNameMappingIngress:
 
     def __call__(self, *ingress_args, **ingress_kwargs):
         # Get the all-keywords version of the arguments (args,kwargs->kwargs)
-        func_kwargs = self.outer_sig.kwargs_from_args_and_kwargs(
-            ingress_args, ingress_kwargs
-        )
+        func_kwargs = self.outer_sig.map_arguments(ingress_args, ingress_kwargs)
         # Modify the keys of func_kwargs so they reflect the inner signature's names
         # That is, map outer names to inner names.
         func_kwargs = dict(
@@ -1023,7 +1019,7 @@ class ArgNameMappingIngress:
 
         # Return an (args,kwargs) pair the respects the inner function's
         # argument kind restrictions.
-        return self.inner_sig.args_and_kwargs_from_kwargs(func_kwargs)
+        return self.inner_sig.mk_args_and_kwargs(func_kwargs)
 
 
 def mk_ingress_from_name_mapper(func, name_mapper: Mapping, *, conserve_kind=False):
@@ -1045,10 +1041,6 @@ def modify_dict_on_cond(d, cond, func):
     return {k: apply_func_on_cond(func, cond, k, v) for k, v in d.items()}
 
 
-def convert_to_PK(kinds):
-    return {name: PK for name in kinds}
-
-
 def convert_VK_to_KO(kinds):
     cond = lambda k, v: v == VK
     func = lambda v: KO
@@ -1056,47 +1048,7 @@ def convert_VK_to_KO(kinds):
     return modify_dict_on_cond(kinds, cond, func)
 
 
-def nice_kinds(func, kinds_modifier=convert_to_PK):
-    """Wraps the func, changing the argument kinds according to kinds_modifier.
-    The default behaviour is to change all kinds to POSITIONAL_OR_KEYWORD kinds.
-    The original purpose of this function is to remove argument-kind restriction
-    annoyances when doing functional manipulations such as:
-
-    >>> from functools import partial
-    >>> isinstance_of_str = partial(isinstance, class_or_tuple=str)
-    >>> isinstance_of_str('I am a string')
-    Traceback (most recent call last):
-      ...
-    TypeError: isinstance() takes no keyword arguments
-
-    Here, instead, we can just get a kinder version of the function and do what we
-    want to do:
-
-    >>> _isinstance = nice_kinds(isinstance)
-    >>> isinstance_of_str = partial(_isinstance, class_or_tuple=str)
-    >>> isinstance_of_str('I am a string')
-    True
-    >>> isinstance_of_str(42)
-    False
-
-    See also: ``i2.signatures.all_pk_signature``
-
-    """
-    from i2 import Sig, call_somewhat_forgivingly
-
-    sig = Sig(func)
-    kinds_modif = kinds_modifier(sig.kinds)
-
-    sig = sig.ch_kinds(**kinds_modif)
-
-    # sig = sig.ch_kinds(**{name: Sig.POSITIONAL_OR_KEYWORD for name in sig.names})
-
-    @wraps(func)
-    def _func(*args, **kwargs):
-        return call_somewhat_forgivingly(func, args, kwargs, enforce_sig=sig)
-
-    _func.__signature__ = sig
-    return _func
+nice_kinds = deprecation_of(kind_forgiving_func, "nice_kinds")
 
 
 def wrap_from_sig(func, new_sig):
@@ -1235,7 +1187,7 @@ def rm_params(
     if not allow_removal_of_non_defaulted_params:
         assert not params_to_remove_that_do_not_have_defaults, (
             f"Some of the params you want to remove don't have defaults: "
-            f'{params_to_remove_that_do_not_have_defaults}'
+            f"{params_to_remove_that_do_not_have_defaults}"
         )
 
     return wrap(
@@ -1259,16 +1211,16 @@ def arg_val_converter_ingress(func, __strict=True, **conversion_for_arg):
     if __strict:
         conversion_names_that_are_not_func_args = conversion_for_arg.keys() - sig.names
         assert not conversion_names_that_are_not_func_args, (
-            'Some of the arguments you want to convert are not argument names '
-            f'for the function: {conversion_names_that_are_not_func_args}'
+            "Some of the arguments you want to convert are not argument names "
+            f"for the function: {conversion_names_that_are_not_func_args}"
         )
 
     @sig
     def ingress(*args, **kwargs):
         # TODO: Make a helper function for this ak -> k -> proc -> ak pattern
-        kwargs = sig.kwargs_from_args_and_kwargs(args, kwargs)
+        kwargs = sig.map_arguments(args, kwargs)
         kwargs = dict(convert_dict_values(kwargs, conversion_for_arg))
-        args, kwargs = sig.args_and_kwargs_from_kwargs(kwargs)
+        args, kwargs = sig.mk_args_and_kwargs(kwargs)
         return args, kwargs
 
     return ingress
@@ -1283,8 +1235,8 @@ class ArgValConverterIngress:
                 conversion_for_arg.keys() - sig.names
             )
             assert not conversion_names_that_are_not_func_args, (
-                'Some of the arguments you want to convert are not argument names '
-                f'for the function: {conversion_names_that_are_not_func_args}'
+                "Some of the arguments you want to convert are not argument names "
+                f"for the function: {conversion_names_that_are_not_func_args}"
             )
         self.sig = sig
         self.conversion_for_arg = conversion_for_arg
@@ -1292,9 +1244,9 @@ class ArgValConverterIngress:
 
     def __call__(self, *args, **kwargs):
         # TODO: Make a helper function for this ak -> k -> proc -> ak pattern
-        kwargs = self.sig.kwargs_from_args_and_kwargs(args, kwargs)
+        kwargs = self.sig.map_arguments(args, kwargs)
         kwargs = dict(convert_dict_values(kwargs, self.conversion_for_arg))
-        args, kwargs = self.sig.args_and_kwargs_from_kwargs(kwargs)
+        args, kwargs = self.sig.mk_args_and_kwargs(kwargs)
         return args, kwargs
 
 
@@ -1422,11 +1374,11 @@ def camelize(s):
     >>> camelize('camel_case')
     'CamelCase'
     """
-    return ''.join(ele.title() for ele in s.split('_'))
+    return "".join(ele.title() for ele in s.split("_"))
 
 
 def kwargs_trans_to_extract_args_from_attrs(
-    outer_kwargs: dict, attr_names=(), obj_param='self'
+    outer_kwargs: dict, attr_names=(), obj_param="self"
 ):
     self = outer_kwargs.pop(obj_param)
     arguments_extracted_from_obj = {name: getattr(self, name) for name in attr_names}
@@ -1449,7 +1401,7 @@ def func_to_method_func(
     *,
     method_name=None,
     method_params=None,
-    instance_arg_name='self',
+    instance_arg_name="self",
 ) -> MethodFunc:
     """Get a 'method function' from a 'normal function'. Also known as "methodize".
 
@@ -1623,10 +1575,10 @@ def bind_funcs_object_attrs(
     if isinstance(init_params, str):
         init_params = init_params.split()
 
-    dflt_cls_name = 'FuncsUnion'
+    dflt_cls_name = "FuncsUnion"
     if callable(funcs) and not isinstance(funcs, Iterable):
         single_func = funcs
-        dflt_cls_name = camelize(getattr(single_func, '__name__', dflt_cls_name))
+        dflt_cls_name = camelize(getattr(single_func, "__name__", dflt_cls_name))
         funcs = [single_func]
 
     # If cls not given, make one from the funcs and init_params
@@ -1675,7 +1627,7 @@ def _mk_base_class_for_funcs(cls_name, init_params, funcs):
             )
             class_init_sig = class_init_sig.merge_with_sig(
                 init_params_in_func_sig,
-                default_conflict_method='fill_defaults_and_annotations',
+                default_conflict_method="fill_defaults_and_annotations",
             )
 
     dataclass_fields = list(map(param_to_dataclass_field_tuple, class_init_sig.params))
@@ -1685,7 +1637,10 @@ def _mk_base_class_for_funcs(cls_name, init_params, funcs):
 
 
 def bind_funcs_object_attrs_old(
-    funcs, init_params=(), *, cls=None,
+    funcs,
+    init_params=(),
+    *,
+    cls=None,
 ):
     """Transform one or several functions into a class that contains them as methods
     sourcing specific arguments from the instance's attributes.
@@ -1740,10 +1695,10 @@ def bind_funcs_object_attrs_old(
     if isinstance(init_params, str):
         init_params = init_params.split()
 
-    dflt_cls_name = 'FuncsUnion'
+    dflt_cls_name = "FuncsUnion"
     if callable(funcs) and not isinstance(funcs, Iterable):
         single_func = funcs
-        dflt_cls_name = camelize(getattr(single_func, '__name__', dflt_cls_name))
+        dflt_cls_name = camelize(getattr(single_func, "__name__", dflt_cls_name))
         funcs = [single_func]
 
     if not isinstance(cls, type):
@@ -1973,7 +1928,7 @@ class Wrapx(_Wrap):
 
     def __call__(self, *args, **kwargs):
         try:
-            _kwargs = self.sig.kwargs_from_args_and_kwargs(args, kwargs)
+            _kwargs = self.sig.map_arguments(args, kwargs)
             # TODO: Consider call_forgivingly(self.ingress, *args, **kwargs)
             #  because call_forgivingly(self.ingress, **_kwargs) doesn't allow
             #  same ingress functions to be used for Wrap and Wrapx
@@ -1987,7 +1942,7 @@ class Wrapx(_Wrap):
             # more specific error messages
             # We don't do this in the first run so that we don't incur the validation
             # overhead on every call.
-            _kwargs = self.sig.kwargs_from_args_and_kwargs(args, kwargs)
+            _kwargs = self.sig.map_arguments(args, kwargs)
             func_args, func_kwargs = _validate_ingress_output(
                 call_forgivingly(self.ingress, **_kwargs)
             )
@@ -2003,8 +1958,8 @@ def _validate_ingress_output(ingress_output):
         or not isinstance(ingress_output[1], dict)
     ):
         raise IngressValidationError(
-            f'An ingress function should return a (tuple, dict) pair. '
-            f'This ingress function returned: {ingress_output}'
+            f"An ingress function should return a (tuple, dict) pair. "
+            f"This ingress function returned: {ingress_output}"
         )
     return ingress_output
 
@@ -2033,18 +1988,18 @@ def _mk_composite_sig(ingress_sig, egress_sig, caller_sig):
 def _init_caller(caller):
     if caller is None:
         caller = _default_caller
-        caller_sig = Sig('func args kwargs')  # sig with three inputs
+        caller_sig = Sig("func args kwargs")  # sig with three inputs
     else:
         caller_sig = Sig(caller)
         if len(caller_sig) < 3:
             raise CallerValidationError(
-                f'A caller must have at least three arguments: '
-                f'{caller} signature was {caller_sig}'
+                f"A caller must have at least three arguments: "
+                f"{caller} signature was {caller_sig}"
             )
         if not _all_kinds_are_keyword_only_or_variadic_keyword(caller_sig):
             raise CallerValidationError(
-                f'A caller must have at least three arguments'
-                f'{caller} signature was {caller_sig}'
+                f"A caller must have at least three arguments"
+                f"{caller} signature was {caller_sig}"
             )
     return caller, caller_sig
 
@@ -2053,7 +2008,7 @@ def _init_egress(func_sig, egress):
     if egress is None:
         egress = _default_egress
         # signature with a single 'output' arg and func_sig's return_annotation
-        egress_sig = Sig('output', return_annotation=func_sig.return_annotation)
+        egress_sig = Sig("output", return_annotation=func_sig.return_annotation)
         return_annotation = func_sig.return_annotation
     else:
         egress_sig = Sig(
@@ -2122,9 +2077,7 @@ def partialx(
     f = partial(func, *args, **kwargs)
     if _rm_partialize:
         sig = Sig(func)
-        partialized = list(
-            sig.kwargs_from_args_and_kwargs(args, kwargs, allow_partial=True)
-        )
+        partialized = list(sig.map_arguments(args, kwargs, allow_partial=True))
         sig = sig - partialized
         f = sig(partial(f, *args, **kwargs))
     if _allow_reordering:
@@ -2136,9 +2089,7 @@ def partialx(
     return f
 
 
-def move_params_to_the_end(
-    func: Callable, names_to_move: Union[Callable, Iterable[str]]
-):
+def move_params_to_the_end(func: Callable, names_to_move: Callable | Iterable[str]):
     """
     Choose args from func, according to choice_args_func and move them
     to the right
@@ -2156,8 +2107,8 @@ def move_params_to_the_end(
     if callable(names_to_move):
         names_to_move = names_to_move(func)
     assert isinstance(names_to_move, Iterable), (
-        f'names_to_move must be an iterable of names '
-        f'or a callable producing one from a function. Was {names_to_move}'
+        f"names_to_move must be an iterable of names "
+        f"or a callable producing one from a function. Was {names_to_move}"
     )
 
     names = Sig(func).names
@@ -2191,6 +2142,7 @@ def move_names_to_the_end(names, names_to_move_to_the_end):
 # --------------------------------------------------------------------------------------
 # smart defaults
 
+
 # IDEA: Could make SmartDefault have an out and __call__, working like a meshed.FuncNode
 @dataclass
 class SmartDefault:
@@ -2199,15 +2151,15 @@ class SmartDefault:
 
     def __repr__(self):
         func = self.func_computing_default
-        func_name = getattr(func, '__name__', str(func))
+        func_name = getattr(func, "__name__", str(func))
         if self.original_default is empty:
-            return f'SmartDefault({func_name})'
+            return f"SmartDefault({func_name})"
         else:
             dflt = self.original_default
             if isinstance(dflt, str):
                 return f"SmartDefault({func_name}, '{dflt}')"
             else:
-                return f'SmartDefault({func_name}, {dflt})'
+                return f"SmartDefault({func_name}, {dflt})"
 
 
 # PATTERN: Yet another "meshed dict completion"
@@ -2290,7 +2242,11 @@ def _compute_new_sigs(func, /, **smart_defaults):
 
 @double_up_as_factory
 def add_smart_defaults(
-    func=None, *, _only_if_name_missing=True, _allow_overwrites=False, **smart_defaults,
+    func=None,
+    *,
+    _only_if_name_missing=True,
+    _allow_overwrites=False,
+    **smart_defaults,
 ):
     """Add smart defaults to function.
 
